/* $Id: error-handler.c,v 0.2 1997/08/30 18:00:52 yoichi v0_70 $
 *
 * dhcpcd - DHCP client daemon -
 * Copyright (C) 1996 - 1997 Yoichi Hariguchi <yoichi@fore.com>
 *
 * Dhcpcd is an RFC2131 and RFC1541 compliant DHCP client daemon.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * EC: Needed to do some rewriting as VMS does not have syslog.h and
 * associated routines.  Error messages from DHCPC are sent to the operator
 * log file (SYS$SYSTEM:OPERATOR.LOG) and any terminal enabled to receive
 * network operator requests.  Caution: sometimes if the DHCP server is 
 * unreachable you may get hundreds of thousands of messages.  Feel free to
 * tinker with the code to disable messages, send messages to another file,
 * or whatever you prefer.
 *
 * Everett Chong
 * echong@my-deja.com
 */

#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include "error-handler.h"

#ifdef VMS
#define LOG_ERR 3
#include <stdlib.h>
#include <ssdef.h>
#include <opcdef.h>
#include <descrip.h>
#include <starlet.h>
#else
#include <syslog.h>
#endif

#define MAXLINE 1024

int DebugFlag;

static void	errorPrint(int flag, const char *fmt, va_list ap);
static void	sendLog(int flag, int priority, const char *fmt, va_list ap);


#ifdef VMS
char input_buffer[256];
$DESCRIPTOR(input_desc, input_buffer);
#endif


static void
errorPrint(int flag, const char *fmt, va_list ap)
{
	int		errnoOrg;
	char	buf[MAXLINE];

	errnoOrg = errno;
	vsprintf(buf, fmt, ap);
	if ( flag ) {
		sprintf(buf+strlen(buf), ": %s", strerror(errnoOrg));
	}
	strcat(buf, "\n");
	fflush(stdout);			/* in case stdout and stderr are the same */
	fputs(buf, stderr);
	fflush(NULL);			/* flushes all stdio output streams */
}

void
errSysExit(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	errorPrint(1, fmt, ap);
	va_end(ap);
	exit(1);
}

void
errSysRet(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	errorPrint(1, fmt, ap);
	va_end(ap);
}

void
errQuit(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	errorPrint(0, fmt, ap);
	va_end(ap);
	exit(1);
}


void
errMsg(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	errorPrint(0, fmt, ap);
	va_end(ap);
}

static void
sendLog(int flag, int priority, const char *fmt, va_list ap)
{
	int  errnoOrg;
	char buf[MAXLINE];
#ifdef VMS
        char buf1[MAXLINE];
        int  length = 0;
        struct OPC request; 
        struct dsc$descriptor req_desc;
#endif

	errnoOrg = errno;
	vsprintf(buf, fmt, ap);
	if ( flag ) {
		sprintf(buf+strlen(buf), ": %s", strerror(errnoOrg));
	}
	strcat(buf, "\n");
	if ( DebugFlag ) {
		fflush(stdout);
		fputs(buf, stderr);
		fflush(NULL);
	} else {
#ifdef VMS
                sprintf(buf1, "DHCP Client: Priority %d: %s", priority, buf);
                length = strlen(buf1);
                input_desc.dsc$a_pointer = buf1;
                request.opc$b_ms_type = OPC$_RQ_RQST;
                request.opc$b_ms_target = OPC$M_NM_CENTRL | OPC$M_NM_NTWORK;
                request.opc$l_ms_rqstid = 0;
                memcpy(&request.opc$l_ms_text,input_desc.dsc$a_pointer,length);
                req_desc.dsc$w_length = length + 8;
                req_desc.dsc$a_pointer = (char *) &request;
                sys$sndopr(&req_desc,0);
#else
		syslog(priority, buf);
#endif
	}
}

#ifndef VMS
void
logOpen(const char *ident, int option, int facility)
{
	if ( !DebugFlag ) {
		openlog(ident, option, facility);
	}
}
#endif

void
logSysExit(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	sendLog(1, LOG_ERR, fmt, ap);
	va_end(ap);
#ifndef VMS
	closelog();
#endif
	exit(2);
}

void
logSysRet(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	sendLog(1, LOG_ERR, fmt, ap);
	va_end(ap);
}

void
logQuit(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	sendLog(0, LOG_ERR, fmt, ap);
	va_end(ap);
#ifndef VMS
	closelog();
#endif
	exit(2);
}

void
logRet(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	sendLog(0, LOG_ERR, fmt, ap);
	va_end(ap);
}
